package test.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.json.client.JSONNumber;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.InlineLabel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextBox;

public class Page implements EntryPoint {
    private static interface Handler {
        public void processError(String message);
        public void processSuccess(Object body);
    }
    private void sendHttpRequest(String url, RequestBuilder.Method method, String body, final Handler h) {
        try {
            RequestBuilder builder = new RequestBuilder(method, url);
            @SuppressWarnings("unused")
            Request request = builder.sendRequest(body, new RequestCallback() {
                public void onError(Request request, Throwable exception) {
                    h.processError(exception.getMessage());
                }
                public void onResponseReceived(Request request, Response response) {
                    if (response.getStatusCode() == 200) {
                        String json = response.getText();
                        Object o = JSON2Object(json);
                        h.processSuccess(o);
                    } else {
                        h.processError(response.getStatusText());
                    }
                }
            });
        } catch (RequestException ex) {
            h.processError(ex.getMessage());
        }
    }
    public void onModuleLoad() {
        final TextBox tb1 = new TextBox();
        final TextBox tb2 = new TextBox();
        final TextBox tb3 = new TextBox();
        final Button addbtn = new Button("Add");
        final Grid addgrd = new Grid(6, 2);
        addbtn.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                int v1 = Integer.parseInt(tb1.getText());
                int v2 = Integer.parseInt(tb2.getText());
                JSONObject o = new JSONObject();
                o.put("v1", new JSONNumber(v1));
                o.put("v2", new JSONNumber(v2));
                sendHttpRequest("http://localhost:81/add.php", RequestBuilder.POST, o.toString(), new Handler() {
                    public void processError(String message) {
                        addgrd.setWidget(5, 0, new Label("!!!!" + message));
                    }
                    public void processSuccess(Object body) {
                        V3Holder v3 = (V3Holder)body;
                        tb3.setText(Integer.toString(v3.getV3()));
                    }
                });
            }
        });
        sendHttpRequest("http://localhost:81/text.php", RequestBuilder.GET, null, new Handler() {
            public void processError(String message) {
                addgrd.setWidget(5, 0, new Label(message));
            }
            public void processSuccess(Object body) {
                TextHolder text = (TextHolder)body;
                addgrd.setWidget(0, 0, new Label(text.getText()));
            }
        });
        addgrd.setWidget(1, 0, new InlineLabel("First number: "));
        addgrd.setWidget(1, 1, tb1);
        addgrd.setWidget(2, 0, new InlineLabel("Second number: "));
        addgrd.setWidget(2, 1, tb2);
        addgrd.setWidget(3, 0, new InlineLabel("Result number: "));
        addgrd.setWidget(3, 1, tb3);
        addgrd.setWidget(4, 0, addbtn);
        RootPanel.get("content").add(addgrd);
    }
    private final native Object JSON2Object(String json) /*-{ return eval('(' + json + ')'); }-*/;
}

class TextHolder extends JavaScriptObject {
    protected TextHolder() {
    }
    public final native String getText() /*-{ return this.text; }-*/;
}

class V3Holder extends JavaScriptObject {
    protected V3Holder() {
    }
    public final native int getV3() /*-{ return this.v3; }-*/;
}

